# Generated by Django 3.2.16 on 2023-01-17 16:57

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0019_device_redundancy_group_data_migration'),
        ('nautobot_cable_utils', '0005_alter_cabletemplate_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='cabletemplate',
            name='procurement_ident',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='cabletemplate',
            name='supplier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='cable_templates', to='dcim.manufacturer'),
        ),
    ]
